package com.galaxis.manatee.configuration;

import com.galaxis.manatee.util.ThreadUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.SchedulingConfigurer;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;
import org.springframework.scheduling.config.ScheduledTaskRegistrar;

import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.ThreadPoolExecutor;


@Configuration
public class ScheduleConfig implements SchedulingConfigurer {
    private static final Logger log = LoggerFactory.getLogger(ScheduleConfig.class);

    @Value("${schedule.pool.core:64}")
    public int corePoolSize;

    @Override
    public void configureTasks(ScheduledTaskRegistrar taskRegistrar) {
        log.info("任务调度线程池的核心线程数: {}", corePoolSize);
        ThreadPoolExecutor executor = new ScheduledThreadPoolExecutor(
                corePoolSize
                , new ThreadUtil.PrefixThreadFactory("schedule-service-")
                , new ThreadPoolExecutor.DiscardOldestPolicy());
        // 设置线程池处理
        taskRegistrar.setScheduler(executor);
    }
}
