package com.galaxis.manatee.task;

import com.galaxis.manatee.dao.ChuanyunUserCompanyDao;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunUserCompanyDO;
import com.galaxis.manatee.service.LogCheckService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @author zcj
 * @version 0.1
 * @date 2021/3/4 5:19 上午
 */
@Slf4j
@Service
public class LogCheckTask {

    private final LogCheckService logCheckService;
    private final ChuanyunUserCompanyDao chuanyunUserCompanyDao;

    public LogCheckTask(LogCheckService logCheckService, ChuanyunUserCompanyDao chuanyunUserCompanyDao) {
        this.logCheckService = logCheckService;
        this.chuanyunUserCompanyDao = chuanyunUserCompanyDao;
    }

    /**
     * 定时任务每5天执行一次
     * 定时进行日志工时核对任务
     * 1.遍历每个用户，核对原始数据和标准化每日工时
     * 2.遍历每个用户，核对每周、每月工时准确性
     * 3.遍历每个项目核对每个项目总工时
     */
    @Scheduled(fixedDelay = 432000000L, initialDelay = 3600000L)
    private void checkLod() {
        //核对原始数据和标准化每日工时
//                logCheckService.checkDayWorkHour(chuanyunUserCompanyDO.getUserId());
//                logCheckService.checkWeekWorkHour(chuanyunUserCompanyDO.getUserId());
//                logCheckService.checkMonthWorkHour(chuanyunUserCompanyDO.getUserId());

        List<String> allUserIdList = chuanyunUserCompanyDao.findAll().stream().map(ChuanyunUserCompanyDO::getUserId).collect(Collectors.toList());
        List<String> disUserIdList = allUserIdList.stream().distinct().collect(Collectors.toList());
        logCheckService.checkDayWorkHourTimeAfter(disUserIdList, 90);
        logCheckService.checkWeekWorkHour(disUserIdList);
        logCheckService.checkMonthWorkHour(disUserIdList);
    }
}
