package com.galaxis.manatee.Test.project;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunGroupProjectDO;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunFindAllBizDTO;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunSaveDTO;
import com.galaxis.manatee.entity.chuanyun.dto.Filter;
import com.galaxis.manatee.manager.ChuanYunManager;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.util.StringUtils;

import java.util.List;

@Slf4j
@SpringBootTest
public class ClosePDTest {

    @Autowired
    private ChuanYunManager chuanYunManager;

    @Test
    void closePD() {
        var objectMapper = new ObjectMapper();
        var start = 0;
        var flag = true;
        var pageSize = 100;
        List<String> m = List.of("F0000005_2,研发二开", "F0000075_5,异常", "F0000075_5,已结项", "F0000075_5,已关闭");
        while (flag) {

            //从氚云查询数据
            var filter = Filter.instance(start, start + pageSize, true, Filter.AND, m);
            ChuanyunFindAllBizDTO<Object> chuanyunFindAllResponse = null;
            try {
                chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunGroupProjectDO.SCHEMA_CODE, filter);
            } catch (JsonProcessingException e) {
                e.printStackTrace();
            }
            if (chuanyunFindAllResponse.getReturnData() == null) {
                flag = false;
                continue;
            }
            if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                start += pageSize;
            } else {
                flag = false;
            }

            //转化为POJO
            List<ChuanyunGroupProjectDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
            });
            result.forEach(chuanyunGroupProjectDO -> {
                log.info("项目号: {}, 项目名称: {}, 项目状态: {}", chuanyunGroupProjectDO.getProjectCode(), chuanyunGroupProjectDO.getProjectName(), chuanyunGroupProjectDO.getProjectState());
                try {
//                    boolean b = updateProject(chuanyunGroupProjectDO.getObjectId());
//                    if (b){
//                        log.info("success");
//                    }else {
//                        log.info("fail");
//                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            });

        }
        log.info("完成");
    }

    /**
     * 更新项目
     */
    private boolean updateProject(String objectId) throws Exception {
        ObjectMapper objectMapper = new ObjectMapper();
        GroupProjectStateUpdateDTO groupProjectStateUpdateDTO = new GroupProjectStateUpdateDTO();
        groupProjectStateUpdateDTO.setProjectState("未维护");
        String jsonStr = objectMapper.writeValueAsString(groupProjectStateUpdateDTO);
        ChuanyunSaveDTO update = chuanYunManager.update(ChuanyunGroupProjectDO.SCHEMA_CODE, objectId, jsonStr);
        return update.getSuccessful();
    }


    @Data
    static class GroupProjectStateUpdateDTO {
        /**
         * 字符串表示
         * 未进场
         * 已进场
         * 已上线
         * 进行中
         * 已初验
         * 已终验
         * 已结项
         * 已暂停
         * 已关闭
         * 异常（新增隐藏状态，主要处理重复项目号的问题）
         */
        @JsonProperty("F0000075")
        private String projectState;
    }


    @Test
    void emptyProjectState() {
        var objectMapper = new ObjectMapper();
        var start = 0;
        var flag = true;
        var pageSize = 100;
        while (flag) {

            //从氚云查询数据
            var filter = Filter.instance(start, start + pageSize, true);
            ChuanyunFindAllBizDTO<Object> chuanyunFindAllResponse = null;
            try {
                chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunGroupProjectDO.SCHEMA_CODE, filter);
            } catch (JsonProcessingException e) {
                e.printStackTrace();
            }
            if (chuanyunFindAllResponse.getReturnData() == null) {
                flag = false;
                continue;
            }
            if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                start += pageSize;
            } else {
                flag = false;
            }

            //转化为POJO
            List<ChuanyunGroupProjectDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
            });
            result.forEach(chuanyunGroupProjectDO -> {
                if (!StringUtils.hasLength(chuanyunGroupProjectDO.getProjectState())) {
                    log.info("项目号: {}, 项目名称: {}, 项目状态: {}", chuanyunGroupProjectDO.getProjectCode(), chuanyunGroupProjectDO.getProjectName(), chuanyunGroupProjectDO.getProjectState());
                    try {
                        boolean b = updateProject(chuanyunGroupProjectDO.getObjectId());
                        if (b) {
                            log.info("success");
                        } else {
                            log.info("fail");
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }

            });

        }
        log.info("完成");
    }

}
