package com.galaxis.manatee.Test.workHour;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.galaxis.manatee.dao.ChuanyunSelfWorkHourDao;
import com.galaxis.manatee.dao.ChuanyunUserCompanyDao;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunMemberHourDO;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunSelfWorkHourDO;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunUserCompanyDO;
import com.galaxis.manatee.entity.chuanyun.dto.Filter;
import com.galaxis.manatee.manager.ChuanYunManager;
import com.galaxis.manatee.service.LogService;
import com.galaxis.manatee.service.LogStandardService;
import com.galaxis.manatee.service.LogUpdateService;
import com.galaxis.manatee.task.WorkHourStatistics;
import com.galaxis.manatee.util.TimeUtil;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Slf4j
@SpringBootTest
public class Standard {

    @Autowired
    private LogService logService;

    @Autowired
    private LogStandardService logStandardService;

    @Autowired
    private ChuanyunSelfWorkHourDao chuanyunSelfWorkHourDao;

    @Autowired
    private ChuanyunUserCompanyDao chuanyunUserCompanyDao;

    @Autowired
    private WorkHourStatistics workHourStatistics;

    private LogUpdateService logUpdateService;
    private ChuanYunManager chuanYunManager;

    @Autowired
    public void setLogUpdateService(LogUpdateService logUpdateService) {
        this.logUpdateService = logUpdateService;
    }

    @Autowired
    public void setChuanYunManager(ChuanYunManager chuanYunManager) {
        this.chuanYunManager = chuanYunManager;
    }

    @Test
    void standard() {
        logService.standardAllSelfCost();
    }

    @Test
    void updateAll() {
        logService.updateAllChuanyunSelfCost();
    }

    @Test
    void updateAllDay() {
        logService.updateAllChuanyunSelfCost();
    }

    @Test
    void standardByUserId() {

        List<ChuanyunSelfWorkHourDO> userAllSelfWorkHours = chuanyunSelfWorkHourDao.findByUserId("28f1291d-b322-4983-9893-cc7f1742fcc2");
        Map<LocalDate, Map<String, Map<String, Map<String, BigDecimal>>>> map = userAllSelfWorkHours.stream().filter(selfWorkHourDO ->
                StringUtils.hasLength(selfWorkHourDO.getProjectId()) &&
                        StringUtils.hasLength(selfWorkHourDO.getProjectType()) &&
                        StringUtils.hasLength(selfWorkHourDO.getStatus()) &&
                        selfWorkHourDO.getStandardWorkHour() != null &&
                        selfWorkHourDO.getDayLogDate() != null).collect(
                Collectors.groupingBy(chuanyunSelfWorkHourDO -> LocalDate.of(chuanyunSelfWorkHourDO.getDayLogDate().getYear(), chuanyunSelfWorkHourDO.getDayLogDate().getMonth(), 1),
                        Collectors.groupingBy(ChuanyunSelfWorkHourDO::getProjectId,
                                Collectors.groupingBy(ChuanyunSelfWorkHourDO::getProjectType,
                                        Collectors.groupingBy(ChuanyunSelfWorkHourDO::getStatus, Collectors.reducing(BigDecimal.ZERO, ChuanyunSelfWorkHourDO::getStandardWorkHour, BigDecimal::add)
                                        )))));


    }

    @Test
    void user() {
        List<String> userIdList = chuanyunUserCompanyDao.findAll().stream().map(ChuanyunUserCompanyDO::getUserId).filter(StringUtils::hasLength).collect(Collectors.toList());
        int i = userIdList.indexOf("28f1291d-b322-4983-9893-cc7f1742fcc2");
        ArrayList<String> userId2 = new ArrayList<>();
        for (int j = i; j < userIdList.size(); j++) {
            String s = userIdList.get(j);
            userId2.add(s);
        }
        userId2.parallelStream().forEach(s -> {
                    logStandardService.standardSelfCostByUserId(s);
                    log.info("标准化用户工时完成userId: {}", s);
                }
        );
    }

    @Test
    void standardSelfWorkHour() {
        List<String> userIdList = chuanyunUserCompanyDao.findAll().stream().map(ChuanyunUserCompanyDO::getUserId).filter(StringUtils::hasLength).toList();
        List<String> userIdList2 = userIdList.stream().distinct().toList();
        userIdList2.forEach(logStandardService::standardHalfYearWorkHourByUserId);
    }



    @Test
    void standardAll() {
        workHourStatistics.standardAllWorkHour();
    }

    @Test
    void pushAll() {
        workHourStatistics.pushAllWorkHour();
    }

    @Test
    void standardSelfWorkHourOne() {
        // 3d94c30e-66f1-46e8-a2a3-fe32aa9a071d
        logStandardService.standardSelfWorkHourByUserId("91c295cc-6de9-4a27-abd9-e3d94a5115c4");
    }

    @Test
    void pushOne() {
        logUpdateService.updateChuanyunSelfWorkHourListByUserId("91c295cc-6de9-4a27-abd9-e3d94a5115c4");
    }

    @Test
    void saveOne() {
        ObjectMapper objectMapper = new ObjectMapper();
        try {
            List<String> matchers = new ArrayList<>();
            matchers.add("F0000001_2,ef3e45e8-006f-4a25-8f40-d5832d6898ad");
            matchers.add("F0000002_2,be8724e2-169e-49a1-8806-bd7b3631894c");
            matchers.add("F0000003_2," + TimeUtil.formatDateTime(LocalDateTime.of(2022, 1, 15, 8, 0, 0)));
            matchers.add("F0000005_2,实施");
            var filter = Filter.instance(0, Integer.MAX_VALUE, true, "And", matchers);
            var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunMemberHourDO.SCHEMA_CODE, filter);
            if (chuanyunFindAllResponse.getReturnData() != null) {
                List<ChuanyunMemberHourDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                });
            }

        } catch (Exception e) {
            log.error(e.getMessage());
        }
    }


}
